##############################
# MySQL Driver for E-Blah    #
##############################
# Version 2 Alpha RW         #
##############################

#### How to use this module #############################
# $x = SQL->new('table','UNIQUEID');                    #
# $x->columns('columns you want returned');             #
# $x->where('where it is');                             #
# $x->select(@send_the_variables_for_other_statements); #
#########################################################

use DBI;
package SQL;

sub sqlconnect {
	$stats = {
		_selects => 0,
		_inserts => 0,
		_deletes => 0,
		_updates => 0,
		_stats => undef
	};

	$dbh = DBI->connect("DBI:mysql:database=$main::forumdbdb;host=$main::forumdbserver","$main::forumdbuser","$main::forumdbpass") or error($DBI::errstr);
}

sub error {
	print "Content-type: text/html\n\n";
	print <<"EOT";
<div style="font-size: 12px; font-family: Verdana, Arial">
 <h1>Error</h1>
 <p>We're sorry, but there was an error with the database.</p>
 <p><textarea rows="4" cols="50">$_[0]</textarea></p>
 <p><a href="mailto:$main::eadmin">Contact the forum administrator</a>.</p>
</div>
EOT
	exit;
}

sub new {
    my($class,$table,$varval,$debug) = @_;
    my $self = {
        _table   => $table,
        _where   => undef,
        _columns => '*',
		_order   => undef,
        _limit   => 1,
		_into    => undef,
		_set     => undef,
		_debug   => $debug || undef
    };

	$self->{_varval} = $varval eq '' ? int(rand(time)) : $varval;

	@{"_$self->{_varval}"} = ();
    bless($self,$class);
    return($self);
}

sub columns {
	my(@columns) = @_;
	my($self) = $_[0];
	my($sql);

	shift @columns;

	if($columns[0] eq '') { $self->{_columns} = '*'; return(); }

	foreach(@columns) {
		$sql .= "`$_`,";
	}

	$sql =~ s/,\Z//g;

	$self->{_columns} = $sql;
}

sub into {
	my($self) = $_[0];
	my(@into) = @_;
	my($sql);

	shift @into;

	foreach(@into) {
		$sql .= "`$_`,";
	}

	$sql =~ s/,\Z//g;

	$self->{_into} = $sql;
}

sub set {
	my($self) = $_[0];
	my(@into) = @_;
	my($sql,$set,$addsub);

	shift @into;

	foreach(@into) {
		($set,$addsub) = split(/\|/,$_);

		$sql .= "`$set` = ?," if($addsub eq '');
		$sql .= "$set=`$set`$addsub," if($addsub ne '');
	}

	$sql =~ s/,\Z//g;

	$self->{_set} = $sql;
}

sub where {
	my(@where) = @_;
	my($self) = $_[0];
	my($sql,$name,$cond,$cond2);

	shift @where;

	foreach(@where) {
		($name,$cond,$cond2,$cond3,$cond4,$cond5) = split(/\|/,$_);
		$cond = '='   if($cond eq '');
		$cond2 = 'AND' if($cond2 eq '');
		if($cond3) { $name = "$cond3(`$name`)"; }
			else { $name = "`$name`"; }

		$sql .= "$cond4$name $cond ?$cond5 $cond2 ";
	}

	$sql =~ s/ $cond2 \Z//g;

	$self->{_where} = $sql;
}

sub limit {
	my($self,$limit,$limit2) = @_;

	if($limit eq '') { $self->{_limit} = ''; }
		else {
			# Prevent injection: make sure it's a #
			if($limit !~ s~^(\.?\d+\.?\d*)\Z~$1~g) { $limit = 0; }
			if($limit !~ s~^(\.?\d+\.?\d*)\Z~$1~g) { $limit2 = 15; }
		
			$self->{_limit} = "$limit";
			$self->{_limit} .= ", $limit2";
		}
}

sub col_array {
	my($self) = $_[0];
	return(@{"_$self->{_varval}"});
}

sub stats {
	return($stats->{_inserts},$stats->{_updates},$stats->{_selects},$stats->{_deletes},$stats->{_stats});
}

sub order {
	my($self) = $_[0];
	my(@order) = @_;
	shift @order;

	foreach(@order) {
		my($order,$type) = split(/\|/,$_);
		$type = " $type" if($type ne '');
		$self->{_order} .= "`$order`$type, ";
	}

	$self->{_order} =~ s/, \Z//g;
}

# Actual database interaction:
sub sqlclose {
	$dbh->disconnect();
}

sub insert {
	my($self) = $_[0];
	my(@query) = @_;
	my($questions,$sql);
	shift @query;

	foreach(@query) { $questions .= "?,"; }
	$questions =~ s/,\Z//g;

	$sql = "INSERT INTO `$main::forumprefix$self->{_table}` ($self->{_into}) VALUES ($questions)";

	error($sql) if($self->{_debug});

	$sth = $dbh->prepare($sql);
	eval { $data = $sth->execute(@query) or error($sth->errstr."\n\n$sql\n\n$count\n\n"); };
	if($@) { error($@); }
	$sth->finish;

	++$stats->{_inserts};
	# $stats->{_stats} .= "<i>INSERT</i> ($self->{_into}) <strong>$self->{_table}</strong></strong><br />";

	#return($dbh->last_insert_id('','','','',''));
	return($sth->{mysql_insertid});
}

sub update {
	my($self) = $_[0];
	my(@query) = @_;
	shift @query;

	$sql = "UPDATE `$main::forumprefix$self->{_table}` SET $self->{_set} WHERE $self->{_where}";

	error($sql) if($self->{_debug});

	$sth = $dbh->prepare($sql);
	eval { $affected = $sth->execute(@query) or error($sth->errstr."\n\n$sql\n\n"); };
	if($@) { error($@); }
	$sth->finish;

	++$stats->{_updates};

	return($affected);
	# $stats->{_stats} .= "<i>UPDATE</i> $self->{_set} FROM <strong>$self->{_table}</strong> WHERE <strong>$self->{_where}</strong><br />";
}

sub delete {
	my($self) = $_[0];
	my(@query) = @_;
	shift @query;

	$sql = "DELETE FROM `$main::forumprefix$self->{_table}` WHERE $self->{_where}";

	error($sql) if($self->{_debug});

	$sth = $dbh->prepare($sql);
	eval { $sth->execute(@query) or error($sth->errstr."\n$sql"); };
	if($@) { error($@); }
	$sth->finish;

	++$stats->{_deletes};
	# $stats->{_stats} .= "<i>DELETE</i> FROM <strong>$self->{_table}</strong> WHERE <strong>$self->{_where}</strong> (@query)<br />";
}

sub select {
	my(@query) = @_;
	my($self) = $_[0];
	my($sql,$sth,$data,$varname,$varval,%sqltemp);

	shift @query;

	$sql = "SELECT $self->{_columns} FROM `$main::forumprefix$self->{_table}` ";
	$sql .= "WHERE $self->{_where} " if($self->{_where});
	$sql .= "ORDER BY $self->{_order} " if($self->{_order});
	$sql .= "LIMIT $self->{_limit}" if($self->{_limit});

	error("$sql\n\n@query") if($self->{_debug});

	$sth = $dbh->prepare($sql);
	eval { $sth->execute(@query) or error($sth->errstr."\n$sql"); };
	if($@) { error($@); }

	while($data = $sth->fetchrow_hashref()) {
		while(($key,$value) = each(%$data)) {
			if($data->{$self->{_varval}} eq '') { $data->{$self->{_varval}} = $self->{_varval}; }
			$sqltemp{$data->{$self->{_varval}}}{$key} = $value;
		}
		push(@{"_$self->{_varval}"},$data->{$self->{_varval}});
	}

	$sth->finish();

	++$stats->{_selects};
	++$stats->{"_t$self->{_table}"};
	$stats->{_stats} .= qq~$stats->{"_t$self->{_table}"} -> SELECT $self->{_columns} FROM <strong>$self->{_table}</strong> WHERE <strong>$self->{_where}</strong> (@query)<br />~;

	return(%sqltemp);
}

sub count {
	my(@query) = @_;
	my($self) = $_[0];
	my($sql,$sth,$data,$varname,$varval,%sqltemp);

	shift @query;

	$sql = "SELECT COUNT($self->{_columns}) FROM `$main::forumprefix$self->{_table}` ";
	$sql .= "WHERE $self->{_where} " if($self->{_where});
	$sql .= "ORDER BY $self->{_order} " if($self->{_order});
	$sql .= "LIMIT $self->{_limit}" if($self->{_limit});

	error("$sql | @query") if($self->{_debug});

	$sth = $dbh->prepare($sql);
	eval { $sth->execute(@query) or error($sth->errstr."\n$sql"); };
	if($@) { error($@); }
	$data = $sth->fetchrow_hashref();
	$sth->finish();
	++$stats->{_selects};
	return($data->{"COUNT($self->{_columns})"});
}

sub empty {
	my($self) = $_[0];

	$dbh->do("DELETE FROM `$main::forumprefix$self->{_table}`");
}

sub trans {
	my($self,$trans) = @_;

	$sth = $dbh->prepare($trans);
	eval { $sth->execute() or error($sth->errstr."\n$trans"); };
	if($@) { error($@); }
	return(1);
}
1;